<!-- Hide from older browsers
//////////////////////////////////////////////////////////////
//
//	Copyright 2000 Greg Restall
//	Free for public non-commercial use and modification
//	as long as this header is kept intact and unmodified.
//	Please email Greg.Restall@mq.edu.au with questions or 
//      suggestions.
//
//////////////////////////////////////////////////////////////
//
//The Building Blocks of Formulas.
//Atoms
Ps = {output:'p'}; Qs = {output:'q'}; Rs = {output:'r'}; Ss = {output: 's'};
Zero = {output:'0'}; One = {output:'1'};
//and Connectives
Not = {output:''}; 
Imp = {output:'>'}; And = {output:'&'}; Or = {output:'v'}; Iff = {output:'='};
//To Make a New formula, call new Formula(Not,[frm]), or 
//new Formula(And,[frm1,frm2]), etc.
function Formula(symbol, arg){ this.symbol = symbol; this.arg = arg; }
//Formulas we want around.
PP = new Formula(Ps,[]); QQ = new Formula(Qs,[]); RR = new Formula(Rs,[]); SS = new Formula(Ss,[]);
leafZero = new Formula(Zero,[]); leafOne = new Formula(One,[]);
//Converting a Formula into a String
function formula_to_string(frm) {
    if (frm.arg.length == 0) return frm.symbol.output;
    else if (frm.arg.length == 1) return frm.symbol.output + 
    formula_to_string(frm.arg[0]);
    else return "(" + formula_to_string(frm.arg[0]) +  
    frm.symbol.output + formula_to_string(frm.arg[1]) + ")";
}  
//Converting a String into a Formula
//Returns "failed" if the string is not a formula, and a formula otherwise.
function string_to_formula(s) {
    // s is a Propositional Atom
    if (s=="p" | s=="P") { return PP }
    else if (s=="q" | s=="Q") { return QQ }
    else if (s=="r" | s=="R") { return RR }
	else if (s=="s" | s=="S") { return SS }
    // Binary Connective at Depth Zero   
     var ss = split_string(s);
     var ls = s.length;
     if (ss != "failed") {
	var subformula0 = string_to_formula(ss[1]);
	var subformula1 = string_to_formula(ss[2]);
	if(subformula0 == "failed") { return "failed" }
	else if(subformula1 == "failed") { return "failed" } else {
	    return new Formula(ss[0],[subformula0,subformula1])
          }     
     }
    // Unary Connective at Front
    else if (s.substring(0,1) == "")
      { subformula = string_to_formula(s.substring(1,ls));
	  if(subformula == "failed") { return "failed" } 
	  else { return new Formula(Not,[subformula]) }
      } 
    // Binary Connective at Depth One
    else if (ls > 2 && s.substring(0,1) == "(" && s.substring(ls-1,ls)==")")
    {
        var sis = split_string(s.substring(1,ls-1));
	if (sis != "failed") {
	    var subformula1 = string_to_formula(sis[1]);
	    var subformula2 = string_to_formula(sis[2]);
	    if(subformula1 == "failed") { return "failed" }
	    else if(subformula2 == "failed") { return "failed" } else {
		return new Formula(sis[0],[subformula1,subformula2])
	    }
	} else { return "failed" }
    }   
    else { return "failed" }  
}
// evaluate takes a formula and a row and evaluates the 
// formula on that row.  It returns a "formula" in which 
// each connective or atom is replaced with a truth value.
//
// A row is a number from 0 to 7.  The value of p is its 
// least significant bit, q its next significiant bit, and
// r its most significant.
function evaluate(frm,row) {
    if (frm == SS) { if (row == 0 | row == 2 | row == 4 | row == 6 | row == 8 | row == 10 | row == 12 | row == 14) { return leafOne }
    else { return leafZero } }
    else if (frm == RR) { if (row == 0 | row == 1 | row == 4 | row == 5 | row == 8 | row == 9 | row == 12 | row == 13) { return leafOne }
     else { return leafZero } }
    else if (frm == QQ) { if (row == 0 | row == 1 | row == 2 | row == 3 | row == 8 | row == 9 | row == 10 | row == 11) { return leafOne } 
    else { return leafZero } }
	else if (frm == PP) { if (row == 0 | row == 1 | row == 2 | row == 3 | row == 4 | row == 5 | row == 6 | row == 7) { return leafOne } 
    else { return leafZero } }
    // Evaluate Negations
    else if (frm.symbol == Not) {
	var subformula = evaluate(frm.arg[0],row);
	if (subformula.symbol.output == '1') {
	    return new Formula(Zero,[subformula])
	} else { return new Formula(One,[subformula]) } }
    // Evaluate Conjunctions
    else if(frm.symbol == And) {
	var subformula0 = evaluate(frm.arg[0],row);
	var subformula1 = evaluate(frm.arg[1],row);
	if (subformula0.symbol.output == '1' && subformula1.symbol.output == '1') {
	    return new Formula(One,[subformula0,subformula1])
	} else { return new Formula(Zero,[subformula0,subformula1]) } }
    // Evaluate Disjunctions
    else if(frm.symbol == Or) {
	var subformula0 = evaluate(frm.arg[0],row);
	var subformula1 = evaluate(frm.arg[1],row);
	if (subformula0.symbol.output == '1' | subformula1.symbol.output == '1') {
	    return new Formula(One,[subformula0,subformula1])
	} else { return new Formula(Zero,[subformula0,subformula1]) } }
    // Evaluate Conditionals
    else if(frm.symbol == Imp) {
	var subformula0 = evaluate(frm.arg[0],row);
	var subformula1 = evaluate(frm.arg[1],row);
	if (subformula0.symbol.output == '1' && subformula1.symbol.output == '0') {
	    return new Formula(Zero,[subformula0,subformula1])
	} else { return new Formula(One,[subformula0,subformula1]) } }
    // Evaluate Biconditionals
    else if(frm.symbol == Iff) {
	var subformula0 = evaluate(frm.arg[0],row);
	var subformula1 = evaluate(frm.arg[1],row);
	if (subformula0.symbol.output == subformula1.symbol.output) {
	    return new Formula(One,[subformula0,subformula1])
	} else { return new Formula(Zero,[subformula0,subformula1]) } }
}
// split_string attempts to split a string at a binary connective at depth 
// zero.  If it can, it returns an array of [connective, firstbit, secondbit]
// If it can't, it returns "failed".
function split_string(s) {
    var ls = s.length; var depth = 0;    
    for(i=0;i<ls;i++) {
	var c = s.substring(i,i+1)
	if (c == "(") { depth++ } 
	else if (c == ")") { depth-- }
	else if (c == "&" && depth==0 && i>0 && i<ls) {
	    return [And, s.substring(0,i), s.substring(i+1,ls)] }
	else if (c == "v" && depth==0 && i>0 && i<ls) {
	    return [Or,s.substring(0,i),s.substring(i+1,ls)] }
	else if (c == ">" && depth==0 && i>0 && i<ls) {
	    return [Imp,s.substring(0,i),s.substring(i+1,ls)] }
	else if (c == "=" && depth==0 && i>0 && i<ls) {
	    return [Iff,s.substring(0,i),s.substring(i+1,ls)] } 
	} return "failed";
}
//strip_spaces Strips a string of spaces
function strip_spaces(s) {
var ls = s.length
    var s_without_spaces = ""
    for (i=0;i<ls;i++) {
	if (s.substring(i,i+1) != " ") 
	   { s_without_spaces = s_without_spaces + s.substring(i,i+1) } 
    }
    return s_without_spaces
}
//Here is the main function, making a truth table from a formula.
function make_table(form) {
    //Clear the table
    for (i=2;i<(30*17)+2;i++) { form.elements[i].value = ""}
    //Strip the spaces from the string.
    var input_string = strip_spaces(form.formula.value);
    //Convert it to a formula.
    var the_formula = string_to_formula(input_string);
    //If it isn't a formula, throw an alert and restart.
    if (the_formula == "failed") {
	alert('Lo siento. No puedo entender la frmula\r'
	+'que has introducido ' + input_string + '.\r' + 'Por favor, intntalo de nuevo.') 
    } else { var the_formulastring = formula_to_string(the_formula);	
    //Check the formula string for length.
    if (the_formulastring.length > 30) {
	alert('Desgraciadamente, este guin no puede crear tablas para frmulas de ms de 30 caracteres, parntesis incluidos. Por favor, intntalo con una frmula ms corta.');
    } else {
    //put the formula at the top of the table.
    form.formula.value = the_formulastring;
    for (i=0;i<the_formulastring.length;i++) {
	form.elements[i+2].value = the_formulastring.substring(i,i+1)
    }
    //put the rest of the table in.
    for (j=0;j<16;j++) {
	var the_value = evaluate(the_formula,j);
	var the_value_string = formula_to_string(the_value);
	for (i=0;i<the_formulastring.length;i++) {
	    if (the_value_string.substring(i,i+1) == "1" | the_value_string.substring(i,i+1) == "0") {
		form.elements[i+2+((j+1)*30)].value = the_value_string.substring(i,i+1);
//		form.elements[i+2+((j+1)*30)].class = "test";
	    }
	}
    }
    }
    }    
}
function put_in_box(s) {
    document.forms.truthtable.formula.value=s
}
// Stop hiding -->
// JavaScript Document